#!/usr/bin/perl -w
use strict;
use warnings;

#Adjust TSS location based on CADRE and GenBank annotation files

my $infile1=$ARGV[0]; #annotation file (CADRE)
my $infile2=$ARGV[1]; #annotation file (GENBANK)
my $infile3=$ARGV[2]; #TSS BED file
my $outfile=$ARGV[3]; #output new bed file with new tss position

open (REFC, "$infile1") or die "\nERROR: Could not open $infile1\n";
open (REFG, "$infile2") or die "\nERROR: Could not open $infile2\n";
open (BED, "$infile3") or die "\nERROR: Could not open $infile3\n";
open (OUT, ">$outfile") or die "\nERROR: Could not open $outfile\n";


my (@bedf,@bedc,@tssf,@tssc,@out);
#bedf, bedc: array from tss bed file
#out: output
#tssf,tssc: array of tss data (tss position, tss distance, gene name)
my($direc,$gstart,$gene,@gf,@gc);
#direc: gene strand (forward or complement)
#gstart: whole gene start position
#gf: list of gene postions on forward strand
#gc: list of gene postions on complement strand

while (<BED>) { #input the tss bed file as a array of array
	my @ts=split();
   if($ts[5] eq '+'){push @bedf,[@ts];push @tssf,[$ts[1]];}
   if($ts[5] eq '-'){push @bedc,[@ts];push @tssc,[$ts[1]];}
}


#get gene start position and name for forword and complement
while (<REFC>){
    if(/FT   gene/){
          my @tt=/(\d+)/g;
          if(/complement/){
            $direc='c';
            $gstart=$tt[1];
           }
          else{
          	$direc='f';
            $gstart=$tt[0];
          }
		  	 <REFC>;
          $_=<REFC>;
          if($_=~/locus_tag="(.+)"/){$gene=$1;}
          if(($direc eq 'f') && ($gene=~/ANIA/)){push @gf,[$gstart,$gene];}
          elsif(($direc eq 'c') && ($gene=~/ANIA/)){push @gc,[$gstart,$gene];}
    }
}


#associate the tss position to the nearest gene
my $Lf=@tssf;
my $Lc=@tssc;
my $f=0;
foreach my $i (0..$Lf-1){
    until($tssf[$i][0]<=$gf[$f][0] ){ $f++;}
    my $d1=$tssf[$i][0]-$gf[$f-1][0];
    my $d2=$tssf[$i][0]-$gf[$f][0];
    if (abs($d1)<=abs($d2)){push @{$tssf[$i]},($d1,$gf[$f-1][1]);}
    else {push @{$tssf[$i]},($d2,$gf[$f][1]);}
}

my $c=0;
foreach my $i (0..$Lc-1){
    until($tssc[$i][0]<=$gc[$c][0] ){ $c++;}
    my $d1=$tssc[$i][0]-$gc[$c-1][0];
    my $d2=$tssc[$i][0]-$gc[$c][0];
    if (abs($d1)<=abs($d2)){push @{$tssc[$i]},($d1,$gc[$c-1][1]);}
    else {push @{$tssc[$i]},($d2,$gc[$c][1]);}
}

my (%tf,%tc);#hash array with key of genename
foreach my $i (0..$Lf-1){push @{$tf{$tssf[$i][2]}},[$i,$tssf[$i][1]];}
foreach my $i (0..$Lc-1){push @{$tc{$tssc[$i][2]}},[$i,$tssc[$i][1]];}


#ajust tss position in terms of position of genes in GenBank and relative distance
$f=0;
$c=0;
while (<REFG>){
    if(/\sgene\s/){
          my @tt=/(\d+)/g;
          if(/complement/){
            $direc='c';
            $gstart=$tt[1];
           }
          else{
          	$direc='f';
            $gstart=$tt[0];
          }
          while(<REFG>){
           	 if(/locus_tag="(.+)"/){
                $gene=$1;
            	 if(($direc eq 'f') && ($f<$Lf) && (exists $tf{$gene})){
                     foreach my $t (@{$tf{$gene}}){
                         my $dis=@{$t}[1];
                         my $idx=@{$t}[0];
                         my $pos=$gstart+$dis;
                         $bedf[$idx][1]=$bedf[$idx][2]=$pos;
                         $f++;
                     }
	              }
	              if(($direc eq 'c') && ($c<$Lc) && (exists $tc{$gene})){
                     foreach my $t (@{$tc{$gene}}){
                         my $dis=@{$t}[1];
                         my $idx=@{$t}[0];
                         my $pos=$gstart+$dis;
                         $bedc[$idx][1]=$bedc[$idx][2]=$pos;
                         $c++;
                     }
	              }
                 last;
              }
          }
    }
    if($f>=$Lf && $c>=$Lc){last;}
}


#output
foreach my $i (0..$Lf-1){print OUT "@{$bedf[$i]}\n";}
foreach my $i (0..$Lc-1){print OUT "@{$bedc[$i]}\n";}


close (REFC) or die "\nERROR: Could not close $infile1\n";
close (REFG) or die "\nERROR: Could not close $infile2\n";
close (BED) or die "\nERROR: Could not close $infile3\n";
close (OUT) or die "\nERROR: Could not close $outfile\n";
